package se.nekotronic.androidbouncer;

import java.util.ArrayList;

import android.os.Bundle;
import android.app.Activity;
import android.text.format.Time;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

public class BouncerActivity extends Activity implements OnClickListener {
	public static final int PORT = 2003;

	private TextView output_area;
	private Button start_button;
	private Button stop_button;

	private ServerThread server_thread;
	private ArrayList<ClientTalkerThread> client_talker_threads = new ArrayList<ClientTalkerThread>();

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_bouncer);
		output_area = (TextView)findViewById(R.id.output_area);
		output_area.setMovementMethod(new ScrollingMovementMethod());
		start_button = (Button)findViewById(R.id.start_button);
		start_button.setOnClickListener(this);
		stop_button = (Button)findViewById(R.id.stop_button);
		stop_button.setOnClickListener(this);

		output_area.setText("");
		print("AndroidPinger: Appen startar.");
	}
	
	private synchronized void stop_all_threads() {
		print("Stopping all threads (if any)");
		if (server_thread != null) {
			server_thread.please_stop();
			server_thread = null;
		}
		for (ClientTalkerThread c : client_talker_threads) {
			c.please_stop();
		}
		// client_talker_threads.clear(); -- The threads remove themselves!
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		stop_all_threads();
	}

	@Override
	public void onClick(View view) {
		Log.d("*** OnePinger", "Button clicked: view = " + view.toString() + "\n");
		if (view == start_button) {
			if (server_thread != null) {
				warning("Already running.");
			}
			else {
				server_thread = new ServerThread(this);
				server_thread.start();
			}
		}
		else if (view == stop_button) {
			stop_all_threads();
			// this.finish();
		}
	} // onClick

	public void print(String message) {
		Log.d("*** OnePinger", "message: " + message);
		Time now = new Time();
		now.setToNow();
		String timeString = now.format("%H:%M:%S");
		final String line = timeString + ": " + message + "\n";
		output_area.post(new Runnable() {
			@Override
			public void run() {
				output_area.setText(output_area.getText() + line);
				// Hr borde vi scrolla automatiskt
				output_area.invalidate();
			}
		});
	} // print
	
	public void warning(String message) {
		print("Warning! " + message);
	}

	public synchronized void add_client(ClientTalkerThread c) {
		client_talker_threads.add(c);
	}

	public synchronized void client_has_disconneted(ClientTalkerThread c) {
		client_talker_threads.remove(c);
	}
} // BouncerActivity
