package se.nekotronic.androidbouncer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.Enumeration;

public class ServerThread extends Thread {
	private BouncerActivity parent;
	private boolean quit = false;
	private ServerSocket server_socket;

	public ServerThread(BouncerActivity parent) {
		this.parent = parent;
	}

	public void please_stop() {
		quit = true;
		try {
			server_socket.close();
		} catch (IOException e) {
			// Not much we can do here
			parent.print("Failed to close server socket: " + e);
		}
	}

	public void run() {
		try {
			parent.print("Ntverksstatus...");
			showNetworkStatus();

			parent.print("ppnar server-socketen...");
			server_socket = new ServerSocket(parent.PORT);
			parent.print("Server-socketen: " + server_socket);
			parent.print("Servern lyssnar...");

			while(!quit) {
				parent.print("Vntar p uppkoppling frn klient...");
				// Blocks until a connection occurs:
				Socket socket = server_socket.accept();
				parent.print("Uppkoppling frn klient accepterad.");
				parent.print("Den nya socketen: " + socket);
				final BufferedReader from_client = new BufferedReader(new InputStreamReader(socket.getInputStream()));
				final PrintWriter to_client = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);
				// true: PrintWriter is line buffered

				ClientTalkerThread c = new ClientTalkerThread(parent, socket, from_client, to_client);
				c.start();
				parent.add_client(c);
			}
		} catch (java.net.BindException e) {
			parent.print("java.net.BindException: " + e);
			e.printStackTrace();
		} catch (IOException e) {
			parent.print("IOException: " + e);
			e.printStackTrace();
		} catch (Exception e) {
			parent.print("Some other Exception: " + e);
			e.printStackTrace();
		}
	} // run

	private void showNetworkStatus() {
		try {
			for (Enumeration<NetworkInterface> is = NetworkInterface.getNetworkInterfaces(); is.hasMoreElements();) {
				NetworkInterface i = is.nextElement();
				// parent.print("Interface: " + i.getName() + ", " + (i.isUp() ? "up" : "down")); -- Requires API 9
				parent.print("Interface: " + i.getName());
				for (Enumeration<InetAddress> ip = i.getInetAddresses(); ip.hasMoreElements();) {
					InetAddress a = ip.nextElement();
					parent.print("  Address: " + a.toString());
				}
			}
		}
		catch (SocketException e) {
			parent.print("Caught exception: " + e);
		}
	} // showNetworkStatus
} // ServerThread
